@@ -54,13 +54,18 @@
  * ------------------------------------------------------------------- */
 
 #define HEADERS()
-
 #include "headers.h"
 #include "Server.hpp"
 #include "List.h"
 #include "Extractor.h"
 #include "Reporter.h"
 #include "Locale.h"
+#ifdef HAVE_SCHED_SETSCHEDULER
+#include <sched.h>
+#endif
+#ifdef HAVE_MLOCKALL
+#include <sys/mman.h>
+#endif
 
 /* -------------------------------------------------------------------
  * Stores connected socket and socket info.
@@ -100,48 +105,162 @@ void Server::Run( void ) {
     long currLen; 
     max_size_t totLen = 0;
     struct UDP_datagram* mBuf_UDP  = (struct UDP_datagram*) mBuf; 
-
     ReportStruct *reportstruct = NULL;
+    int running;
+    static struct timeval watchdog;
+
+#if HAVE_DECL_SO_TIMESTAMP
+    // Structures needed for recvmsg
+    // Use to get kernel timestamps of packets
+    struct sockaddr_storage srcaddr;
+    struct iovec iov[1];
+    iov[0].iov_base=mBuf;
+    iov[0].iov_len=mSettings->mBufLen;
+    struct msghdr message;
+    message.msg_iov=iov;
+    message.msg_iovlen=1;
+    message.msg_name=&srcaddr;
+    message.msg_namelen=sizeof(srcaddr);
+    char ctrl[CMSG_SPACE(sizeof(struct timeval))];
+    struct cmsghdr *cmsg = (struct cmsghdr *) &ctrl;
+    message.msg_control = (char *) ctrl;
+    message.msg_controllen = sizeof(ctrl);
+#endif
 
     reportstruct = new ReportStruct;
     if ( reportstruct != NULL ) {
         reportstruct->packetID = 0;
         mSettings->reporthdr = InitReport( mSettings );
+	running=1;
+	// Set the socket timeout to 1/2 the report interval
+	if (mSettings->mInterval) {
+#ifdef WIN32
+	  // Windows SO_RCVTIMEO uses ms
+	    DWORD timeout;
+	    timeout = (mSettings->mInterval / 2.0) * 1e3;
+#else
+	  // Linux SO_RCVTIMEO uses timeval
+	    struct timeval timeout;
+	    double intpart, fractpart, half;
+	    half = mSettings->mInterval / 2;
+	    fractpart = modf(half, &intpart);
+	    timeout.tv_sec = (int) (intpart);
+	    timeout.tv_usec = (int) (fractpart * 1e6);
+#endif
+	    if (setsockopt( mSettings->mSock, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(timeout)) < 0 ) {
+		WARN_errno( mSettings->mSock == SO_RCVTIMEO, "socket" );
+	    }
+	}
+#if HAVE_DECL_SO_TIMESTAMP
+        if ( isUDP( mSettings ) ) {
+	    int timestampOn = 1;
+	    if (setsockopt(mSettings->mSock, SOL_SOCKET, SO_TIMESTAMP, (int *) &timestampOn, sizeof(timestampOn)) < 0) {
+		WARN_errno( mSettings->mSock == SO_TIMESTAMP, "socket" );
+	    }
+	}
+#endif
+#ifdef HAVE_SCHED_SETSCHEDULER
+	if ( isRealtime( mSettings ) ) {
+	    struct sched_param sp;
+	    sp.sched_priority = sched_get_priority_max(SCHED_RR); 
+	    // SCHED_OTHER, SCHED_FIFO, SCHED_RR
+	    if (sched_setscheduler(0, SCHED_RR, &sp) < 0)  {
+		perror("Client set scheduler");
+#ifdef HAVE_MLOCKALL
+	    } else if (mlockall(MCL_CURRENT | MCL_FUTURE) != 0) { 
+		// lock the threads memory
+		perror ("mlockall");
+#endif
+	    }
+	}
+#endif
+	gettimeofday( &watchdog, NULL );
         do {
+	    reportstruct->emptyreport=0;
+#if HAVE_DECL_SO_TIMESTAMP
             // perform read 
-            currLen = recv( mSettings->mSock, mBuf, mSettings->mBufLen, 0 ); 
-        
-            if ( isUDP( mSettings ) ) {
+            currLen = recvmsg( mSettings->mSock, &message, 0 );
+	    if (currLen <= 0) {
+		// Socket read timeout or read error
+		reportstruct->emptyreport=1;
+		gettimeofday( &(reportstruct->packetTime), NULL );
+                // End loop on 0 read or socket error
+		// except for socket read timeout
+		if (currLen == 0 || (TimeDifference(reportstruct->packetTime, watchdog) > (mSettings->mAmount / 100)) ||
+#ifdef WIN32
+		    (WSAGetLastError() != WSAEWOULDBLOCK)
+#else
+		    (errno != EAGAIN && errno != EWOULDBLOCK)
+#endif		     
+		    ) {
+		    running = 0;
+		}
+		currLen=0;
+	    }
+
+            if (!reportstruct->emptyreport && isUDP( mSettings ) ) {
                 // read the datagram ID and sentTime out of the buffer 
                 reportstruct->packetID = ntohl( mBuf_UDP->id ); 
                 reportstruct->sentTime.tv_sec = ntohl( mBuf_UDP->tv_sec  );
                 reportstruct->sentTime.tv_usec = ntohl( mBuf_UDP->tv_usec ); 
 		reportstruct->packetLen = currLen;
+		if (cmsg->cmsg_level == SOL_SOCKET &&
+		    cmsg->cmsg_type  == SCM_TIMESTAMP &&
+		    cmsg->cmsg_len   == CMSG_LEN(sizeof(struct timeval))) {
+			memcpy(&(reportstruct->packetTime), CMSG_DATA(cmsg), sizeof(struct timeval));
+		} else {
+		    gettimeofday( &(reportstruct->packetTime), NULL );
+		}
+            }
+#else
+            // perform read 
+            currLen = recv( mSettings->mSock, mBuf, mSettings->mBufLen, 0 );
+	    if (currLen <= 0) {
+		reportstruct->emptyreport=1;
+                // End loop on 0 read or socket error
+		// except for socket read timeout
+		if (currLen == 0 ||
+#ifdef WIN32
+		    (WSAGetLastError() != WSAEWOULDBLOCK)
+#else
+		    (errno != EAGAIN && errno != EWOULDBLOCK)
+#endif		     
+		    ) {
+		    running = 0;
+		}
+		currLen = 0;
+	    }
+            if (!reportstruct->emptyreport && isUDP( mSettings ) ) {
 		gettimeofday( &(reportstruct->packetTime), NULL );
-            } else {
+		reportstruct->packetLen = currLen;
+                // read the datagram ID and sentTime out of the buffer 
+		reportstruct->packetID = ntohl( mBuf_UDP->id ); 
+		reportstruct->sentTime.tv_sec = ntohl( mBuf_UDP->tv_sec  );
+		reportstruct->sentTime.tv_usec = ntohl( mBuf_UDP->tv_usec ); 
+            }
+#endif
+	    if (currLen) {
+		watchdog = reportstruct->packetTime;
 		totLen += currLen;
 	    }
-        
             // terminate when datagram begins with negative index 
             // the datagram ID should be correct, just negated 
             if ( reportstruct->packetID < 0 ) {
                 reportstruct->packetID = -reportstruct->packetID;
-                currLen = -1; 
+                currLen = -1;
+		running = 0; 
             }
 
 	    if ( isUDP (mSettings)) {
 		ReportPacket( mSettings->reporthdr, reportstruct );
-            } else if ( !isUDP (mSettings) && mSettings->mInterval > 0) {
+            } else {
+		// TCP case
                 reportstruct->packetLen = currLen;
                 gettimeofday( &(reportstruct->packetTime), NULL );
                 ReportPacket( mSettings->reporthdr, reportstruct );
             }
-
-
-
-        } while ( currLen > 0 ); 
-        
-        
+        } while (running); 
+                
         // stop timing 
         gettimeofday( &(reportstruct->packetTime), NULL );
         
@@ -205,15 +324,27 @@ void Server::write_UDP_AckFIN( ) {
             hdr->total_len1   = htonl( (long) (stats->TotalLen >> 32) );
             hdr->total_len2   = htonl( (long) (stats->TotalLen & 0xFFFFFFFF) );
             hdr->stop_sec     = htonl( (long) stats->endTime );
-            hdr->stop_usec    = htonl( (long)((stats->endTime - (long)stats->endTime)
-                                              * rMillion));
+            hdr->stop_usec    = htonl( (long)((stats->endTime - (long)stats->endTime) * rMillion));
             hdr->error_cnt    = htonl( stats->cntError );
             hdr->outorder_cnt = htonl( stats->cntOutofOrder );
             hdr->datagrams    = htonl( stats->cntDatagrams );
             hdr->jitter1      = htonl( (long) stats->jitter );
-            hdr->jitter2      = htonl( (long) ((stats->jitter - (long)stats->jitter) 
-                                               * rMillion) );
-
+            hdr->jitter2      = htonl( (long) ((stats->jitter - (long)stats->jitter) * rMillion) );
+            hdr->minTransit1  = htonl( (long) stats->transit.totminTransit );
+            hdr->minTransit2  = htonl( (long) ((stats->transit.totminTransit - (long)stats->transit.totminTransit) * rMillion) );
+            hdr->maxTransit1  = htonl( (long) stats->transit.totmaxTransit );
+            hdr->maxTransit2  = htonl( (long) ((stats->transit.totmaxTransit - (long)stats->transit.totmaxTransit) * rMillion) );
+            hdr->sumTransit1  = htonl( (long) stats->transit.totsumTransit );
+            hdr->sumTransit2  = htonl( (long) ((stats->transit.totsumTransit - (long)stats->transit.totsumTransit) * rMillion) );
+            hdr->meanTransit1  = htonl( (long) stats->transit.totmeanTransit );
+            hdr->meanTransit2  = htonl( (long) ((stats->transit.totmeanTransit - (long)stats->transit.totmeanTransit) * rMillion) );
+            hdr->m2Transit1  = htonl( (long) stats->transit.totm2Transit );
+            hdr->m2Transit2  = htonl( (long) ((stats->transit.totm2Transit - (long)stats->transit.totm2Transit) * rMillion) );
+            hdr->vdTransit1  = htonl( (long) stats->transit.totvdTransit );
+            hdr->vdTransit2  = htonl( (long) ((stats->transit.totvdTransit - (long)stats->transit.totvdTransit) * rMillion) );
+            hdr->cntTransit   = htonl( stats->transit.totcntTransit );
+	    hdr->IPGcnt = htonl( (long) (stats->cntDatagrams / (stats->endTime - stats->startTime)));
+	    hdr->IPGsum = htonl(1);
         }
 
         // write data 
